# Documentation string. I wonder what ever refers to it?

testString = """
http://www.namshub.org:8080/test/manila/message/create?username=""&password=""&sitename=""&subject="test"&body="test%20test%20test"&bodyType="text/plain"&inResponseTo=""&windowInfo=""&rendererInfo=""
"""

__doc__ = """ZopeFish product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

from OFS.DTMLMethod import DTMLMethod

#from ZODB.Transaction import Transaction, get_transaction

import OFS.SimpleItem
import OFS.Folder
import Acquisition
import AccessControl.Role

import re
import string
from xmlrpclib import True, False, Server

from PikeCompatibility import *
from ZopeFishMessageCollection import *
from ZopeFishPreferences import *
from ZopeFishUsers import *

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addZopeFishForm = HTMLFile('zopefishAdd', globals()) 

def manage_addZopeFish(self, id, title='', REQUEST=None):
    """Add a ZopeFish to a folder."""
    if title == '':
        title = 'ZopeFish!'
    self._setObject(id, ZopeFish(id, title, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

"""Step #2: Provide the class for your ZopeFish object."""

class ZopeFishCredits(Acquisition.Implicit):
    """ZopeFishCredits"""
    def __init__(self, id, parent):
        """   """
        self.id = id
        self.title = "ZopeFish Credits"
        self.parent = parent
        self.body = HTMLFile("credits", globals())

class ZopeFish(
    OFS.Folder.Folder,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """ZopeFish object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """
        
    meta_type = 'ZopeFish' # what do people think they're adding? 

    manage_options = ( # what management options are there?
	{'label': 'Edit',               'action': 'manage_main'},
	{'label': 'View',               'action': ''}, # defaults to index_html
	{'label': 'Security',           'action': 'manage_access'},
        {'label': 'Global Preferences', 'action': 'prefs'},
        {'label': 'Users',              'action': 'users'},
        {'label': 'Messages',           'action': 'discuss/manage_main'},
        {'label': 'Credits',            'action': 'credits'},
        ) 

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Change ZopeFishes',       ('manage_edit',)             ),
	('View ZopeFishes',         ('',)                        ),
	)

    def __init__(self, id, title='', REQUEST=None): 
        """initialise a new instance of ZopeFish"""
        self.parent = self
        self.id = id
        self.title = title
        self.template = DTMLMethod()
        self.template.manage_edit(defaultTemplate, "template")
        self.index_html = self.template

        if REQUEST is not None:
            self.location = REQUEST.URL3 + "/" + self.id
            base = self.location[7:]
            index = string.find(base, "/")
            self.locationNoHost = base[index:]
        else:
            self.location = "unknown"

        self.discuss = ZopeFishMessageCollection("discuss", self)
        self.manila = PikeCompatibility("manila", self)
        self.prefs = ZopeFishPreferences("prefs", self)
        self.credits = ZopeFishCredits("credits", self)
        self.users = ZopeFishUserManagement("users", self)

    def allMessages(self):
        """allMessages"""
        return self.discuss.index_html()

    def manage_afterAdd(self, item, container): pass
    def manage_afterClone(self, item): pass
    def manage_beforeDelete(self, item, container): pass
    def self(self): return self

    def objectItems(self, type=None):
        """   """
        return [(self.discuss.id, self.discuss), (self.manila.id, self.manila)]

    def objectValues(self, type=None):
        """   """
        return [self.discuss, self.manila]

    def objectIds(self, type=None):
        """   """
        return [self.discuss.id, self.manila.id]

    def getMember(self, REQUEST):
        """   """
        if REQUEST is not None:
            #return REQUEST.AUTHENTICATED_USER.getUserName()
            return self.users.getLoggedInUsername(REQUEST)
        else:
            return "test member"

    def getStoryInfo(self, REQUEST, subject, body, msgNum, member=None):
        """   """
        if member is None:
            thisMember = self.getMember(REQUEST)
        else:
            thisMember = member

        if REQUEST is None:
            url = self.location
        else:
            url = REQUEST.URL2

        # this is a hack.
        if string.find(url, "/discuss") == -1:
            url = url + "/discuss"

        storyInfo = {}
        storyInfo['body'] = body
        storyInfo['member'] = thisMember
        storyInfo['msgnum'] = msgNum
        storyInfo['postTime'] = ""
        storyInfo['lastUpdate'] = ""
        storyInfo['ctReads'] = 0
        storyInfo['subject'] = subject
        storyInfo['responses'] = []
        storyInfo['URL'] = url + "/" + msgNum
        return storyInfo

    def getSiteInfo(self, REQUEST):
        """   """
        host = REQUEST.HTTP_HOST
        i = string.find(host, ":")
        if i != -1:
            host = host[0:i]

        url = REQUEST.URL2
        if url[-1] != "/":
            url = url + "/"

        siteInfo = {}
        siteInfo['host'] = host
        siteInfo['port'] = REQUEST.SERVER_PORT
        siteInfo['rpcPath'] = self.locationNoHost
        siteInfo['siteName'] = self.zopeFishId()
        siteInfo['displaySiteName'] = self.zopeFishTitle()
        siteInfo['username'] = self.getMember(REQUEST)
        siteInfo['password'] = ""
        siteInfo['URL'] = url
        return siteInfo

    def getPikeServer(self, REQUEST=None, RESPONSE=None):
        """   """
        user = self.users.getUser(REQUEST)
        if user.pikeHost is not None:
            return user.pikeHost + ":" + str(user.pikePort)
        elif REQUEST is not None:
            return Server("http://" + REQUEST.REMOTE_ADDR + ":" + str(user.pikePort))
        else:
            return Server(myPike)

    def zopeFishId(self):
        """zopeFishId"""
        return self.parent.id

    def zopeFishTitle(self):
        """zopeFishTitle"""
        return self.parent.title

    def discussTag(self, tag=None):
        """discussTag"""
        if tag is None:
            tag = "Discussion Group"
        return "<a href=\"" + \
               self.locationNoHost + \
               "/discuss\">" + \
               tag + \
               "</a>"

    def prefsTag(self, tag=None):
        """prefsTag"""
        if tag is None:
            tag = "Global Preferences"
        return "<a href=\"" + \
               self.locationNoHost + \
               "/prefs\">" + \
               tag + \
               "</a>"

    def creditsTag(self, tag=None):
        """creditsTag"""
        if tag is None:
            tag = "ZopeFish Credits"
        return "<a href=\"" + \
               self.locationNoHost + \
               "/credits\">" + \
               tag + \
               "</a>"

    def zopeFishHomeTag(self, tag=None):
        """zopeFishHomeTag"""
        if tag is None:
            tag = "home"
        return "<a href=\"" + \
               self.locationNoHost + \
               "/\">" + \
               tag + \
               "</a>"

    def editTemplateInPikeTag(self, tag=None):
        """editTemplateTag"""
        if tag is None:
            tag = "Edit Template"
        return "<a href=\"" + \
               self.locationNoHost + \
               "/editTemplateInPike\">" + \
               tag + \
               "</a>"

    def editTemplateInPike(self, REQUEST=None, RESPONSE=None):
        """editTemplateInPike"""
        if REQUEST is None:
            return
        body = self.template.read()
        body = re.sub("\r\n", "\n", body)
        body = re.sub("\n", "\r", body)
        itemInfo = {}
        itemInfo['body'] = body
        itemInfo['type'] = 'Template'
        siteInfo = self.getSiteInfo(REQUEST)
        pikeServer = self.getPikeServer(REQUEST, RESPONSE)
        pikeServer.pike.editAdvancedItem(siteInfo, itemInfo)

    body = HTMLFile('index', globals()) # View Interface
    
    manage_main = HTMLFile('zopefishEdit', globals()) # Management Interface
    manage_workspace = manage_main
    
    def manage_edit(self, title, REQUEST=None): 
        """does this really need a doc string?"""
        self.title = title
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "Properties for %s changed." % self.id,
		action = './manage_main',
		)
