from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

from OFS.DTMLMethod import DTMLMethod

#from ZODB.Transaction import Transaction, get_transaction

import OFS.SimpleItem
import OFS.Folder
import Acquisition
import AccessControl.Role

import re
import string
from xmlrpclib import True, False, Server

defaultPikeHost = None
defaultPikePort = 81

class ZopeFishUserRecord:
    """ZopeFishUserRecord"""

    def __init__(self, username, password, email):
        """__init__"""
        self.username = username
        self.password = password
        self.email = email
        self.manager = 0
        self.editor = 0
        self.body = HTMLFile("userDisplay", globals())
        self.pikeHost = defaultPikeHost
        self.pikePort = defaultPikePort

##    def isEditor(self):
##        """isEditor"""
##        return self.editor

##    def isManager(self):
##        """isManager"""
##        return self.manager


class ZopeFishUserManagement(
    OFS.Folder.Folder,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ):
    """ZopeFishUserManagement"""

    body = HTMLFile("users", globals())
    userLogin = HTMLFile("userLogin", globals())
    userList = HTMLFile("userList", globals())
    userAdd = HTMLFile("userAdd", globals())

    def __init__(self, id, parent):
        """__init__"""
        self.id = id
        self.title = "ZopeFish User Management"
        self.parent = parent
        self.users = {}

        self.superuser = ZopeFishUserRecord("superuser", "never used", "never used")
        self.superuser.manager = 1
        self.superuser.editor = 1

    def login(self, username, password, loginRedirect, failedLoginRedirect, REQUEST, RESPONSE):
        """login"""
        if self.users.has_key(username):
            if self.users[username].password == password:
                REQUEST.set("zopeFishCookie", username)
                RESPONSE.setCookie("zopeFishCookie", username,
                                   expires="1/1/2010",
                                   path=self.locationNoHost)
                RESPONSE.redirect(loginRedirect)
                print "successful login:", username
                return
        print "failed:", username
        REQUEST.RESPONSE.redirect(failedLoginRedirect)

    def logoff(self, redirect, REQUEST, RESPONSE):
        """logoff"""
        RESPONSE.expireCookie("zopeFishCookie")
        RESPONSE.redirect(redirect)

    def join(self, username, password, email, loginRedirect, failedLoginRedirect, REQUEST, RESPONSE):
        """join"""
        if self.users.has_key(username):
            print "already have a", username
            RESPONSE.redirect(failedLoginRedirect)
            return
        print "adding", username
        users = self.users
        users[username] = ZopeFishUserRecord(username, password, email)
        self.users = users
        self.login(username, password, loginRedirect, failedLoginRedirect, REQUEST, RESPONSE)

    def loggedIn(self, REQUEST):
        """loggedIn"""
        if self.isZopeManager(REQUEST):
            return 1
        if REQUEST.cookies.has_key("zopeFishCookie"):
            if not self.users.has_key(REQUEST.cookies["zopeFishCookie"]):
                REQUEST.RESPONSE.expireCookie("zopeFishCookie")
                return 0
            else:
                return 1
        else:
            return 0
        return 0

    def isEditor(self, REQUEST):
        """isEditor"""
        if self.loggedIn(REQUEST):
            user = self.users[REQUEST.cookies['zopeFishCookie']]
            return user.editor
        return 0

    def isZopeManager(self, REQUEST):
        """isZopeManager"""
        return REQUEST.AUTHENTICATED_USER.has_role("Manager")

    def isManager(self, REQUEST):
        """isManager"""
        if self.isZopeManager(REQUEST):
            return 1
        if self.loggedIn(REQUEST):
            return self.getUser(REQUEST).manager
        return 0

    def getLoggedInUsername(self, REQUEST):
        """getLoggedInUsername"""
        if self.isZopeManager(REQUEST):
            return self.superuser.username
        if REQUEST.cookies.has_key("zopeFishCookie"):
            return REQUEST.cookies['zopeFishCookie']
        return ""

    def setManager(self, username, REQUEST):
        """setManager"""
        if REQUEST.AUTHENTICATED_USER.has_role("Manager"):
            self.users[username].manager = 1

    def setEditor(self, username, REQUEST):
        """setManager"""
        if REQUEST.AUTHORIZED_USER.has_role("Manager"):
            self.users[username].editor = 1

    def allUsers(self, REQUEST):
        """allUsers"""
        if self.isManager(REQUEST):
            userList = []
            for u in self.users.keys():
                userList.append(self.users[u])
            return userList
        else:
            return []

    def getUser(self, REQUEST):
        """getUser"""
        if self.loggedIn(REQUEST):
            if self.isZopeManager(REQUEST):
                return self.superuser
            return self.users[self.getLoggedInUsername(REQUEST)]
        else:
            return None
